{ **************************************************************************** }
{ *                                                                          * }
{ * SUB CHASE (Mattel Electronics) Simulator                                 * }
{ * by Luca "MADrigal" Antignano, April 2000 (revision: August 2021)         * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;       im_background_overlay_top: TImage;

    btn_power_on:             TImage;       btn_power_top:             TImage;

    btn_up_down:              TImage;       btn_up_top:                TImage;
    btn_left_down:            TImage;       btn_left_top:              TImage;
    btn_right_down:           TImage;       btn_right_top:             TImage;
    btn_down_down:            TImage;       btn_down_top:              TImage;

    btn_fire_up:              TImage;       btn_fire_top:              TImage;

{ * LED SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (score, digits)
    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage;

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;
    im_panel_number_a: TImage; im_panel_number_b: TImage; im_panel_number_c: TImage; im_panel_number_d: TImage; im_panel_number_e: TImage;

    // on-screen items
    im_ship_1_1: TImage; im_ship_2_1: TImage; im_ship_3_1: TImage; im_ship_4_1: TImage; im_ship_5_1: TImage; im_ship_6_1: TImage; im_ship_7_1: TImage;
    im_ship_1_2: TImage; im_ship_2_2: TImage; im_ship_3_2: TImage; im_ship_4_2: TImage; im_ship_5_2: TImage; im_ship_6_2: TImage; im_ship_7_2: TImage;
    im_ship_1_3: TImage; im_ship_2_3: TImage; im_ship_3_3: TImage; im_ship_4_3: TImage; im_ship_5_3: TImage; im_ship_6_3: TImage; im_ship_7_3: TImage;

    im_torpedo_1_1: TImage; im_torpedo_2_1: TImage; im_torpedo_3_1: TImage; im_torpedo_4_1: TImage; im_torpedo_5_1: TImage; im_torpedo_6_1: TImage; im_torpedo_7_1: TImage;
    im_torpedo_1_2: TImage; im_torpedo_2_2: TImage; im_torpedo_3_2: TImage; im_torpedo_4_2: TImage; im_torpedo_5_2: TImage; im_torpedo_6_2: TImage; im_torpedo_7_2: TImage;
    im_torpedo_1_3: TImage; im_torpedo_2_3: TImage; im_torpedo_3_3: TImage; im_torpedo_4_3: TImage; im_torpedo_5_3: TImage; im_torpedo_6_3: TImage; im_torpedo_7_3: TImage;

{ * TIMERS                                                                   * }
    Timer_Game:      TTimer;
    Timer_Game_Over: TTimer;
    Timer_Reprise:   TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All_Ship             (b: boolean);
    procedure Vid_Show_All_Torpedo          (b: boolean);

    procedure Vid_Show_Ship                 (b: boolean);

    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove        (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_power_topMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_up_topMouseDown           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_up_topMouseUp             (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseUp          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_down_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_down_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_fire_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_fire_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_power_down;

    procedure Inp_up_down;                  procedure Inp_up_up;
    procedure Inp_left_down;                procedure Inp_left_up;
    procedure Inp_right_down;               procedure Inp_right_up;
    procedure Inp_down_down;                procedure Inp_down_up;

    procedure Inp_fire_down;                procedure Inp_fire_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Check_Collision;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_RepriseTimer            (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
{}//procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Sub Chase (Mattel Electronics)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';                // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY      = 'Software\MADrigal\Subchase50';                                // registry key

  HINT_CLICK   = 'click or press';
  HINT_PWR_OFF = 'to turn on the game';
  HINT_PWR_ON  = 'to turn off the game';

{}MENU_ITEMS = 12;
  NUM_KEYS = 6;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_gameover,
  pfs_hit,
  pfs_missed,
  pfs_sonar:               PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..2]        of TImage;                        // empty sprites to host score/time digits
  aIm_panel_number:        array[0..14]       of TImage;                        // digits for score

  aIm_Ship,                                                                     // player's ship
  aIm_Torpedo:             array[1..7, 1..3]  of TImage;                        // torpedoes

{ * arrays of mixed types *}
  aKeys:                   array[1..NUM_KEYS] of record                         // keys management (values)
                                                   int:  integer;
                                                   word: word;
                                                 end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  aTorpedoDir:             array[1..7, 1..3]  of record                         // direction of the torpedo (left or right)
                                                   Left,
                                                   Right: boolean;
                                                 end;

  rPosShip,                                                                     // coordinates of the player's ship
  rPosSub:                 record                                               // coordinates of the enemy submarine
                             X,
                             Y: integer;
                           end;

  iMode,                                                                        // 0 = OFF; 1 = ON
  iScore,                                                                       // score = 0-149 (=E9)
  iScoreBonus,                                                                  // bonus points for each destroyed submarine

  iLives,                                                                       // how many mines available on the player's ship

  aSubmarineDir                                                                 // submarine's direction (1=left; 2=right)
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bControls,                                                                    // true while player can actually move the hero
  bArrowPressed,                                                                // true while any arrow key is pressed
  bFirePressed                                                                  // true while the fire button is pressed
                         : boolean;

  // Game variables: loops
  iLoopGame1,                                                                   // main game loop      (sonar beeps at every one or two ticks)
  iLoopGame2,                                                                   // secondary game loop (submarine moves at every 8 ticks of iLoopGame1)
  iLoopGame3,                                                                   // tertiary game loop  (torpedoes move at every 4 ticks of iLoopGame1)

  iLoopGameOver
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}











{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2;

  // numbers
  aIm_panel_number[0]  := im_panel_number_0; aIm_panel_number[1]  := im_panel_number_1; aIm_panel_number[2]  := im_panel_number_2; aIm_panel_number[3]  := im_panel_number_3; aIm_panel_number[4]  := im_panel_number_4;
  aIm_panel_number[5]  := im_panel_number_5; aIm_panel_number[6]  := im_panel_number_6; aIm_panel_number[7]  := im_panel_number_7; aIm_panel_number[8]  := im_panel_number_8; aIm_panel_number[9]  := im_panel_number_9;
  aIm_panel_number[10] := im_panel_number_a; aIm_panel_number[11] := im_panel_number_b; aIm_panel_number[12] := im_panel_number_c; aIm_panel_number[13] := im_panel_number_d; aIm_panel_number[14] := im_panel_number_e;

  // on-screen items
  aIm_Ship[1, 1] := im_ship_1_1; aIm_Ship[2, 1] := im_ship_2_1; aIm_Ship[3, 1] := im_ship_3_1; aIm_Ship[4, 1] := im_ship_4_1; aIm_Ship[5, 1] := im_ship_5_1; aIm_Ship[6, 1] := im_ship_6_1; aIm_Ship[7, 1] := im_ship_7_1;
  aIm_Ship[1, 2] := im_ship_1_2; aIm_Ship[2, 2] := im_ship_2_2; aIm_Ship[3, 2] := im_ship_3_2; aIm_Ship[4, 2] := im_ship_4_2; aIm_Ship[5, 2] := im_ship_5_2; aIm_Ship[6, 2] := im_ship_6_2; aIm_Ship[7, 2] := im_ship_7_2;
  aIm_Ship[1, 3] := im_ship_1_3; aIm_Ship[2, 3] := im_ship_2_3; aIm_Ship[3, 3] := im_ship_3_3; aIm_Ship[4, 3] := im_ship_4_3; aIm_Ship[5, 3] := im_ship_5_3; aIm_Ship[6, 3] := im_ship_6_3; aIm_Ship[7, 3] := im_ship_7_3;

  aIm_Torpedo[1, 1] := im_torpedo_1_1; aIm_Torpedo[2, 1] := im_torpedo_2_1; aIm_Torpedo[3, 1] := im_torpedo_3_1; aIm_Torpedo[4, 1] := im_torpedo_4_1; aIm_Torpedo[5, 1] := im_torpedo_5_1; aIm_Torpedo[6, 1] := im_torpedo_6_1; aIm_Torpedo[7, 1] := im_torpedo_7_1;
  aIm_Torpedo[1, 2] := im_torpedo_1_2; aIm_Torpedo[2, 2] := im_torpedo_2_2; aIm_Torpedo[3, 2] := im_torpedo_3_2; aIm_Torpedo[4, 2] := im_torpedo_4_2; aIm_Torpedo[5, 2] := im_torpedo_5_2; aIm_Torpedo[6, 2] := im_torpedo_6_2; aIm_Torpedo[7, 2] := im_torpedo_7_2;
  aIm_Torpedo[1, 3] := im_torpedo_1_3; aIm_Torpedo[2, 3] := im_torpedo_2_3; aIm_Torpedo[3, 3] := im_torpedo_3_3; aIm_Torpedo[4, 3] := im_torpedo_4_3; aIm_Torpedo[5, 3] := im_torpedo_5_3; aIm_Torpedo[6, 3] := im_torpedo_6_3; aIm_Torpedo[7, 3] := im_torpedo_7_3;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "ON/OFF" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 37);                             // "Up"     (default: "UP")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 39);                             // "Left"   (default: "LEFT")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 40);                             // "Right"  (default: "RIGHT")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 38);                             // "Down"   (default: "DOWN")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 42);                             // "Fire"   (default: "CTRL")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to NUM_KEYS do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_power_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_PWR_OFF; // "Power Off"

  btn_up_top.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int];             // "Up"
  btn_left_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int];             // "Left"
  btn_right_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int];             // "Right"
  btn_down_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];             // "Down"

  btn_fire_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];             // "Fire"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_9_n2;
{}aMenuItem[9]  := pm_10_fullscreen;
{}aMenuItem[10] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[11] := pm_13_n3;
{}aMenuItem[12] := pm_14_exit;

  // variables init

  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  bControls := false;                                                           // prevents hero movement
  bArrowPressed := false;                                                       // no arrow key is currently being kept pressed
  bFirePressed := false;                                                        // fire button is bot currently being kept pressed

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_gameover); FSOUND_Sample_Free(pfs_hit);
  FSOUND_Sample_Free(pfs_missed);   FSOUND_Sample_Free(pfs_sonar);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All_Ship(b: boolean);
{ displays/hides all ships                                                     }
  var
    i, j: integer;
begin
  for i := 1 to 7 do
    for j := 1 to 3 do
      aIm_Ship[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Torpedo(b: boolean);
{ displays/hides all torpedoes                                                 }
  var
    i, j: integer;
begin
  for i := 1 to 7 do
    for j := 1 to 3 do
      aIm_Torpedo[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_Ship(b: boolean);
{ displays/hides player's ship                                                 }
begin
  aIm_Ship[rPosShip.X, rPosShip.Y].Visible := b;
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ converts score/time into sprites, and displays/hides them                    }
  var
    i: integer;
begin
  for i := 1 to 2 do
    aIm_panel_digit_pos[i].Visible := true;

  if p >= 150 then
    p := 149;

  if b then                                                                     // displays or hides score/game time?
    begin                                                                               // displays
      if p >  9 then aIm_panel_digit_pos[1].Picture := aIm_panel_number[p div 10].picture else aIm_panel_digit_pos[1].Picture := nil; // tens (0 to E=14x)
                     aIm_panel_digit_pos[2].Picture := aIm_panel_number[p mod 10].picture;                                            // units
    end
  else
    for i := 1 to 2 do                                                                  // hides
      aIm_panel_digit_pos[i].Picture := nil;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_power_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ POWER button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_power_down;
end;





procedure TForm1.btn_up_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ UP button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_up_down;
end;





procedure TForm1.btn_up_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ UP button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_up_up;
end;





procedure TForm1.btn_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.btn_down_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ DOWN button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_down_down;
end;





procedure TForm1.btn_down_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ DOWN button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_down_up;
end;





procedure TForm1.btn_fire_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ FIRE button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_fire_down;
end;





procedure TForm1.btn_fire_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ FIRE button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_fire_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_power_down  else                              // Power On/Off

  if Key = aKeys[2].word then Inp_up_down     else                              // Up
  if Key = aKeys[3].word then Inp_left_down   else                              // Left
  if Key = aKeys[4].word then Inp_right_down  else                              // Right
  if Key = aKeys[5].word then Inp_down_down   else                              // Down

  if Key = aKeys[6].word then Inp_fire_down;                                    // Fire
end;





procedure TForm1.FormKeyUp(Sender:TObject; var Key:Word; Shift:TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[2].word then Inp_up_up     else                                // Up
  if Key = aKeys[3].word then Inp_left_up   else                                // Left
  if Key = aKeys[4].word then Inp_right_up  else                                // Right
  if Key = aKeys[5].word then Inp_down_up   else                                // Down

  if Key = aKeys[6].word then Inp_fire_up;                                      // Fire
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_power_down;
{ on/off: PRESS                                                                }
begin
  // sets game mode
  if iMode = 0 then                                                             // Power Off ->
    begin                                                                       //           -> Power On (demo mode)
      iMode := 1;
      btn_power_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_PWR_ON; // "click to turn off"
    end
  else                                                                          // whichever ->
    begin                                                                       //           -> Power Off
      iMode := 0;
      btn_power_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_PWR_OFF; // "click to turn on"
    end;

  Gam_Set_Mode;

  btn_power_on.Visible := iMode = 1;
end;





procedure TForm1.Inp_up_down;
{ up arrow key: PRESS                                                          }
begin
  btn_up_down.Visible := true;

  if (iMode = 1) and bControls and (not bArrowPressed) and (rPosShip.Y > 1) then
    begin
      bArrowPressed := true;

      Vid_Show_Ship(false);
      Dec(rPosShip.Y);
      Vid_Show_Ship(true);

      Gam_Check_Collision;
    end;
end;





procedure TForm1.Inp_up_up;
{ up arrow key: RELEASE                                                        }
begin
  btn_up_down.Visible := false;

  bArrowPressed := false;
end;





procedure TForm1.Inp_left_down;
{ left arrow key: PRESS                                                        }
begin
  btn_left_down.Visible := true;

  if (iMode = 1) and bControls and (not bArrowPressed) and (rPosShip.X > 1) then
    begin
      bArrowPressed := true;

      Vid_Show_Ship(false);
      Dec(rPosShip.X);
      Vid_Show_Ship(true);

      Gam_Check_Collision;
    end;
end;





procedure TForm1.Inp_left_up;
{ left arrow key: RELEASE                                                      }
begin
  btn_left_down.Visible := false;

  bArrowPressed := false;
end;





procedure TForm1.Inp_right_down;
{ right arrow key: PRESS                                                       }
begin
  btn_right_down.Visible := true;

  if (iMode = 1) and bControls and (not bArrowPressed) and (rPosShip.X < 7) then
    begin
      bArrowPressed := true;

      Vid_Show_Ship(false);
      Inc(rPosShip.X);
      Vid_Show_Ship(true);

      Gam_Check_Collision;
    end;
end;





procedure TForm1.Inp_right_up;
{ right arrow key: RELEASE                                                     }
begin
  btn_right_down.Visible := false;

  bArrowPressed := false;
end;





procedure TForm1.Inp_down_down;
{ down arrow key: PRESS                                                        }
begin
  btn_down_down.Visible := true;

  if (iMode = 1) and bControls and (not bArrowPressed) and (rPosShip.Y < 3) then
    begin
      bArrowPressed := true;

      Vid_Show_Ship(false);
      Inc(rPosShip.Y);
      Vid_Show_Ship(true);

      Gam_Check_Collision;
    end;
end;





procedure TForm1.Inp_down_up;
{ down arrow key: RELEASE                                                      }
begin
  btn_down_down.Visible := false;

  bArrowPressed := false;
end;





procedure TForm1.Inp_fire_down;
{ fire button: PRESS                                                           }
begin
  btn_fire_up.Visible := false;

  if (iMode = 1) and bControls and (not bFirePressed) then
    begin
      // stops action
      bFirePressed := true;

      Gam_Stop_All_Timers;
      Snd_Silence;

      bControls := false;

      // hides score and torpedoes temporarily
      Vid_Show_Panel_Score(iScore, false);
      Vid_Show_All_Torpedo(false);

      // decreases player's lives (mines)
      Dec(iLives);

      // lays the mine, checks collision mine/submarine, plays sound, etc.
      if (rPosShip.X = rPosSub.X)  and (rPosShip.Y = rPosSub.Y) then
        begin                                                                   // player's ship is above the submarine
          Inc(iScore, iScoreBonus);                                               // increases the score

          if rPosSub.X < 5 then
            begin                                                                 // sets X coordinate of the submarine...
              rPosSub.X := 7;                                                     // ...on the opposite side of the player's ship
              aSubmarineDir := 1;                                                 // sets the shooting direction of the submarine
            end
          else
            begin                                                                 // sets X coordinate of the submarine...
              rPosSub.X := 1;                                                     // sets the shooting direction of the submarine
              aSubmarineDir := 7;                                                 // ...on the opposite side of the player's ship
            end;

          rPosSub.Y := 2;                                                         // sets Y coordinate of the submarine

          Snd_Play(pfs_hit);                                                      // plays "hit" sound

          iScoreBonus := 5;                                                       // resets the bonus score to the maximum (5 points)

          Timer_Reprise.Interval := 1200;                                         // action reprises...
          Timer_Reprise.Enabled := true;
        end
      else                                                                      // the mine has missed the submarine
        begin
          case iScoreBonus of                                                     // decreases the value of bonus points
            5: iScoreBonus := 3;
            3: iScoreBonus := 2;
            2: iScoreBonus := 1;
          end; // case..of

          Snd_Play(pfs_missed);                                                   // plays "missed" sound

          Timer_Reprise.Interval := 1000;                                         // action reprises...
          Timer_Reprise.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_fire_up;
{ fire button: RELEASE                                                         }
begin
  btn_fire_up.Visible := true;

  bFirePressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
begin
  Timer_Game.Enabled      := false;
  Timer_Game_Over.Enabled := false;
  Timer_Reprise.Enabled   := false;
end;





procedure TForm1.Gam_Check_Collision;
{ a miss is made if the player's ship collides with an enemy torpedo           }
  var
    i, j: integer;
    bCollision: boolean;
begin
  bCollision := false;

  for i := 1 to 7 do
    for j := 1 to 3 do
      if aIm_Torpedo[rPosShip.X, rPosShip.Y].Visible then
        bCollision := true;

  if bCollision then
    begin
      Gam_Stop_All_Timers;
      Snd_Silence;

      bControls := false;

      Vid_Show_Panel_Score(iScore, false);

      for i := 1 to 7 do
        for j := 1 to 3 do
          aIm_Torpedo[i, j].Visible := false;

      iLoopGameOver := 1;
      Timer_Game_Over.Interval := 1;
      Timer_Game_Over.Enabled := true;
    end;
end;





procedure TForm1.Gam_Set_Mode;
{ sets off/game/game over mode, depending on the value of the iMode variable   }
  var
    i, j: integer;
begin
  case iMode of
    0: begin                                                                    // off
         Gam_Stop_All_Timers;
         Snd_Silence;
         bControls := false;

         Vid_Show_All_Ship(false);
         Vid_Show_All_Torpedo(false);
         i := 0; Vid_Show_Panel_Score(i, false);
       end;
    1: begin                                                                    // game
         iScore := 0; Vid_Show_Panel_Score(iScore, true);
         iScoreBonus := 5;
         iLives := 30;

         rPosShip.X := 4; rPosShip.Y := 2; Vid_Show_Ship(true);                   // places player's ship in (4,2)
         rPosSub.X := 7;  rPosSub.Y := 1;                                         // places enemy submarind in (7,1)

         aSubmarineDir := 1;                                                      // submarine moves left

         for i := 1 to 7 do                                                       // resets torpedoes array
           for j := 1 to 3 do
             begin
               aTorpedoDir[i, j].Left := false;
               aTorpedoDir[i, j].Right := false;
             end;

         bControls := true;

         iLoopGame1 := 1;                                                         // sonar
         iLoopGame2 := 1;                                                         // submarine
         iLoopGame3 := 1;                                                         // torpedoes
         Timer_Game.Interval := 205;
         Timer_Game.Enabled := true;
       end;
    2: begin                                                                    // game over
         // does nothing
       end;                                                               
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_GameTimer(Sender: TObject);
  var
    i, j: integer;
    bPlaySonar, bSubmarineMoved, bGenerateTorpedo: boolean;
begin
  // torpedoes movement
  if iLoopGame1 = 1 then
   begin
     // erases torpedoes at the leftmost and rightmost side of the stage
     for j := 1 to 3 do
       begin
         aTorpedoDir[1, j].Left := false;
         aTorpedoDir[7, j].Right := false;
       end;

     // moves torpedoes left
     for i := 2 to 7 do
       for j := 1 to 3 do
         if aTorpedoDir[i, j].Left then
           begin
             aTorpedoDir[i,     j].Left := false;
             aTorpedoDir[i - 1, j].Left := true;
           end;

     // moves torpedoes right
     for i := 6 downto 1 do
       for j := 1 to 3 do
         if aTorpedoDir[i, j].Right then
           begin
             aTorpedoDir[i,     j].Right := false;
             aTorpedoDir[i + 1, j].Right := true;
           end;
   end;

  // sets "sonar" sound sample?
  bPlaySonar := false;

  case iLoopGame1 of
    1: begin
         if (Abs(rPosSub.X - rPosShip.X) = 1)
           or (rPosSub.X = rPosShip.X) then
           bPlaySonar := true;
       end;
    2: begin
         if rPosSub.X = rPosShip.X then
           bPlaySonar := true;

         // changes submarine's direction if the sonar is active
         if bPlaySonar then
           if rPosSub.X < rPosShip.X then
             aSubmarineDir := 1
           else
             aSubmarineDir := 7;
       end;
  end; // case..of

  // loops action (n. 1)
  Inc(iLoopGame1);

  if iLoopGame1 = 3 then
    iLoopGame1 := 1;

  // submarine movement (at every 8 timers clicks)
  if iLoopGame2 = 1 then
    begin
      bSubmarineMoved := false;

      while not bSubmarineMoved do
        case Random(3) of
          0: begin                                                              // submarine moves left or right
               if rPosSub.X < aSubmarineDir then
                 Inc(rPosSub.X)
               else
                 Dec(rPosSub.X);

               bSubmarineMoved := true;
             end;
          1: if rPosSub.Y > 1 then                                              // submarine moves up
               begin
                 Dec(rPosSub.Y);

                 bSubmarineMoved := true;
               end;
          2: if rPosSub.Y < 3 then                                              // submarine moves down
               begin
                 Inc(rPosSub.Y);

                 bSubmarineMoved := true;
               end;
        end; // case..of
    end;

  // loops action (n. 2)
  Inc(iLoopGame2);

  if iLoopGame2 = 9 then
    iLoopGame2 := 1;

  // torpedoes: generation
  bGenerateTorpedo := true;

  for i := 1 to 7 do
    if rPosShip.Y = rPosSub.Y then
      begin
        if (rPosShip.X < rPosSub.X) and aTorpedoDir[i, rPosSub.Y].Right then bGenerateTorpedo := false;
        if (rPosShip.X > rPosSub.X) and aTorpedoDir[i, rPosSub.Y].Left  then bGenerateTorpedo := false;
      end;

  if (not bPlaySonar) and (iLoopGame3 = 1) then                                 // at every 4 loops, if sonar is not beeping
    if (rPosShip.Y = rPosSub.Y) and bGenerateTorpedo then
      if rPosShip.X < rPosSub.X then
        aTorpedoDir[rPosSub.X, rPosSub.Y].Left := true
      else
        aTorpedoDir[rPosSub.X, rPosSub.Y].Right := true;

  // loops action (n. 3)
  Inc(iLoopGame3);

  if iLoopGame3 = 5 then
    iLoopGame3 := 1;

  // torpedoes: visualization and collisions check
  for i := 1 to 7 do
    for j := 1 to 3 do
      aIm_Torpedo[i, j].Visible := aTorpedoDir[i, j].Left or aTorpedoDir[i, j].Right;

  Gam_Check_Collision;

  // submarine is at the stage border
  if rPosSub.X = aSubmarineDir then
    if aSubmarineDir = 1 then
      aSubmarineDir := 7
    else
      aSubmarineDir := 1;

  // plays sound sample
  if bPlaySonar then
    Snd_Play(pfs_sonar);
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ game is over: plays "game over" tune, displays final score and then stops    }
begin
  case iLoopGameOver of
    1: begin
         Snd_Play(pfs_gameover);

         Timer_Game_Over.Interval := 1650;
       end;
    2: begin
         Vid_Show_Panel_Score(iScore, true);

         Timer_Game_Over.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopGameOver);
end;





procedure TForm1.Timer_RepriseTimer(Sender: TObject);
{ action reprises after collision mine/submarine                               }
  var
    i, j: integer;
begin
  if iLives >= 1 then                                                           // any lives left?
    begin                                                                         // game reprises
      Vid_Show_Panel_Score(iScore, true);
      Gam_Check_Collision;

      for i := 1 to 7 do
        for j := 1 to 3 do
          aIm_Torpedo[i, j].Visible := aTorpedoDir[i, j].Left or aTorpedoDir[i, j].Right;

      bControls := true;

      Timer_Game.Interval := 205;
      Timer_Game.Enabled := true;
    end
 else                                                                             // game is over
   begin
     iLoopGameOver := 1;
     Timer_Game_Over.Interval := 1;
     Timer_Game_Over.Enabled := true;
   end;

  Timer_Reprise.Enabled := false;
end;











{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
